C----------------------------------------------------------------------------

C  Module name: PHIGS View Utility.

C  Author: Gareth Williams.

C  Function: This module contains the PHIGS viewing utility.

C  Dependencies:

C  Internal function list: 

C  External function list: 

C  Modification history: (Version), (Date), (name), (Description).

C  1.0, 26th November 1991, G. Williams, First version, based on notes by
C  Karen Wyrwas.

C----------------------------------------------------------------------------

        SUBROUTINE ptkf_vieweditor(wsid, num, stids, vwormt, vwmpmt, 
& vwcplm, xyclpi, bclipi, fclipi)
C /*
C ** \parambegin
C ** \param{INTEGER}{wsid}{workstation identifier}{IN}
C ** \param{INTEGER}{num}{number of structures}{IN}
C ** \param{INTEGER}{stids(*)}{list of structure identifiers}{IN}
C ** \param{REAL}{vwormt(4, 4)}{view orientation matrix}{OUT}
C ** \param{REAL}{vwmpmt(4, 4)}{view mapping matrix}{OUT}
C ** \param{REAL}{vwcplm(6)}{view clipping limits}{OUT}
C ** \param{INTEGER}{xyclpi}{x-y clipping indicator}{OUT}
C ** \param{INTEGER}{bclipi}{back clipping indicator}{OUT}
C ** \param{INTEGER}{fclipi}{front clipping indicator}{OUT}
C ** \paramend
C ** \blurb{This function starts the interactive PHIGS view editor, on
C **  workstation \pardesc{wsid}.}
C */
        INTEGER wsid, num, stids(num)
        REAL vwormt(4, 4)
        REAL vwmpmt(4, 4)
        REAL vwcplm(6)
        INTEGER xyclpi, bclipi, fclipi
        external ptkc_vieweditor !$PRAGMA C(ptkc_vieweditor)
 
        call ptkc_vieweditor(%val(wsid), %val(num), stids, vwormt, 
& vwmpmt, vwcplm, xyclpi, bclipi, fclipi)

        RETURN
        END

        SUBROUTINE ptkf_setvieweditorattrs(menufont, windowfont,
& menucol, menutextcol, windowcol, 
& bannercol, bannertextcol, tlcol, brcol, arrowcol, arrowedgecol)
C /*
C ** \parambegin
C ** \param{INTEGER}{menufont}{menu text font}{IN}
C ** \param{INTEGER}{windowfont}{window text font}{IN}
C ** \param{INTEGER}{menucol}{menu colour index}{IN}
C ** \param{INTEGER}{menutextcol}{menu text colour index}{IN}
C ** \param{INTEGER}{windowcol}{window interior colour index}{IN}
C ** \param{INTEGER}{bannercol}{window banner colour index}{IN}
C ** \param{INTEGER}{bannertextcol}{window banner text colour index}{IN}
C ** \param{INTEGER}{tlcol}{top-left colour index}{IN}
C ** \param{INTEGER}{brcol}{bottom-right colour index}{IN}
C ** \param{INTEGER}{arrowcol}{arrow colour index}{IN}
C ** \param{INTEGER}{arrowedgecol}{arrow edge colour index}{IN}
C ** \paramend
C ** \blurb{This function enables the application to
C ** set the text font and colour attribute values for the menus and windows
C **  used in the PHIGS view editor.}
C */
	INTEGER menufont, windowfont
	INTEGER menucol, menutextcol, windowcol
        INTEGER bannercol, bannertextcol, tlcol, brcol
	INTEGER arrowcol, arrowedgecol
        external ptk_setvieweditorattrs
& !$PRAGMA C(ptk_setvieweditorattrs)

 	call ptk_setvieweditorattrs(%val(menufont), %val(windowfont),
& %val(menucol), %val(menutextcol), 
& %val(windowcol), %val(bannercol), %val(bannertextcol), %val(tlcol),
& %val(brcol), %val(arrowcol), %val(arrowedgecol))

	RETURN
	END

        SUBROUTINE ptkf_inqvieweditorattrs(menufont, windowfont,
& menucol, menutextcol, windowcol, 
& bannercol, bannertextcol, tlcol, brcol, arrowcol, arrowedgecol)
C /*
C ** \parambegin
C ** \param{INTEGER}{menufont}{menu text font}{IN}
C ** \param{INTEGER}{windowfont}{window text font}{IN}
C ** \param{INTEGER}{menucol}{menu colour index}{IN}
C ** \param{INTEGER}{menutextcol}{menu text colour index}{IN}
C ** \param{INTEGER}{windowcol}{window interior colour index}{IN}
C ** \param{INTEGER}{bannercol}{window banner colour index}{IN}
C ** \param{INTEGER}{bannertextcol}{window banner text colour index}{IN}
C ** \param{INTEGER}{tlcol}{top-left colour index}{IN}
C ** \param{INTEGER}{brcol}{bottom-right colour index}{IN}
C ** \param{INTEGER}{arrowcol}{arrow colour index}{IN}
C ** \param{INTEGER}{arrowedgecol}{arrow edge colour index}{IN}
C ** \paramend
C ** \blurb{This function may be used to obtain the text font and 
C ** colour attribute values of menus and windows used in the PHIGS view
C ** editor.}
C */
	INTEGER menufont, windowfont
	INTEGER menucol, menutextcol, windowcol
        INTEGER bannercol, bannertextcol, tlcol, brcol
	INTEGER arrowcol, arrowedgecol
        external ptk_inqvieweditorattrs
& !$PRAGMA C(ptk_inqvieweditorattrs)

 	call ptk_inqvieweditorattrs(menufont, windowfont,
& menucol, menutextcol, 
& windowcol, bannercol, bannertextcol, tlcol,
& brcol, arrowcol, arrowedgecol)

	RETURN
	END

C end of view.f
